package udpServer;

import java.io.IOException;
import java.net.*;

public class UdpServer implements Runnable {
	/**
	 * @author Giovanni Romio, Battistini Luca
	 */
	
    /* Udp Server*/
    private DatagramSocket serverSocket;
    private byte[] receiveData = new byte[1024];
    private byte[] sendData = new byte[1024];
    private InetAddress arduinoIpAddress;
    private int arduinoPort;
    private String lastSentence = "";
    private static final int UDPPORT = 6000;
    private static UdpServer myServer;
    private boolean running = true;
    private String serverAddress = "192.168.1.150";
    

    private UdpServer() throws IOException{
        /*Initialize Udp Server on port 6003*/
        serverSocket = new DatagramSocket(UDPPORT);
    }
    /*Singleton istance*/
    public static UdpServer getIstance() throws IOException{
    	if(myServer == null){
    		myServer = new  UdpServer();
    		
    	}
    	return myServer;
    }

    public void run(){
    	/*Read messages*/
    	while(running)
    	{
    		DatagramPacket receivePacket = new DatagramPacket(receiveData, receiveData.length);
    		try {
    			serverSocket.receive(receivePacket);
    		} catch (IOException e) {
    			e.printStackTrace();
    		}

    		arduinoIpAddress = receivePacket.getAddress();
    		arduinoPort = receivePacket.getPort();
    		String sentence = new String( receivePacket.getData());
    		lastSentence = sentence;      
    	}
    }
    
    public void sendMessage(String Message) throws IOException{
    	/*Message to send*/
    	sendData = Message.getBytes();
    	/*The destination is the IpAdress of the Arduino*/
    	 DatagramPacket sendPacket =
                 new DatagramPacket(sendData, sendData.length, arduinoIpAddress, arduinoPort);
                 serverSocket.send(sendPacket);    	
    }
    
    public String getLastMessage(){
    	return lastSentence;
    }
    
    public String getAddress(){
    	return serverAddress;
    }

    public void stopThread(){
        this.running = false;
    }
    
    /**
     * 
     * @return the ip adress of the current Machine
     * @throws UnknownHostException
     */
    public InetAddress getIpAdress() throws UnknownHostException{
    	return InetAddress.getLocalHost();
    }
    
    /**
     * 
     * @return the port of the UdpServer
     */
    public int getPort(){
    	return UDPPORT;
    }
    
    public InetAddress getDeviceConnected(){
    	return arduinoIpAddress;
    }


}
