package controller;

import java.io.IOException;

import model.BombModel;

public abstract class Controller {
	/**
	 * @author Giovanni Romio
	 * @author Battistini Luca
	 */
	protected UdpServer myServer;
	protected BombModel bomb;

	public Controller() {
		myServer = UdpServer.getIstance();
		bomb = BombModel.getIstance();
	}
	
	/**
	 * Initialize the controller
	 */
	public abstract void init();
	
	/**
	 * This method will be called by Controller-Manager Thread 
	 * @throws IOException
	 */
	public abstract void update() throws IOException;
	
	/**
	 * Attach the view at the controller
	 * @param observer
	 */
	public abstract void attachObserver(PanelObserver observer);

	/**
	 * Send a message to the server
	 */
	public abstract void setBomb(String value);

	public UdpServer getServer() {
		return myServer;
	}

	/**
	 * This interface allow communication between View and Controller
	 */

	public interface PanelObserver {	
		/**
		 * 
		 * @param ip of the the Arduino device
		 */
		void displayIp(String ip);

		/**
		 * 
		 * @param message is the message we want to display in the view, tipically the message received from Arduino
		 */
		void displayUpdate(String message);

		/**
		 * Swich the view
		 * @throws IOException 
		 */
		void setNextPanel();
		

	}

}
