package controller;

import java.io.IOException;
import java.net.*;

public class UdpServer implements Runnable {
	/**
	 * @author Giovanni Romio
	 * @author Battistini Luca
	 */

	/* Udp Server*/
	private DatagramSocket serverSocket;
	private byte[] receiveData; 
	private byte[] sendData;
	private InetAddress arduinoIpAddress;
	private int arduinoPort;
	private String lastSentence = "";
	private static final int UDPPORT = 6000;
	private static UdpServer myServer;
	private boolean running;

	private UdpServer() {
		/*Initialize Udp Server on port 6003*/
		try {
			serverSocket = new DatagramSocket(UDPPORT);
		} catch (SocketException e) {
			e.printStackTrace();
		}
		new Thread(this).start();
		running = true;       
	}
	/*Singleton instance*/
	public static UdpServer getIstance() {
		if(myServer == null){
			myServer = new  UdpServer();
		}
		return myServer;
	}
	/**
	 * Read messages
	 */
	public void run() {
		while(running)
		{	
			receiveData = new byte[1024];
			DatagramPacket receivePacket = new DatagramPacket(receiveData, receiveData.length);
			try {
				serverSocket.receive(receivePacket);
			} catch (IOException e) {
				e.printStackTrace();
			}

			arduinoIpAddress = receivePacket.getAddress();
			arduinoPort = receivePacket.getPort();
			String sentence = new String(receivePacket.getData());
			lastSentence = sentence; 

		}
	}

	public void sendMessage(String Message) {
		/*Message to send*/
		sendData =  new byte[1024];
		sendData = Message.getBytes();
		/*The destination is the IpAdress of the ARDUINO*/
		DatagramPacket sendPacket =
				new DatagramPacket(sendData, sendData.length, arduinoIpAddress, arduinoPort);
		try {
			serverSocket.send(sendPacket);
		} catch (IOException e) {
			e.printStackTrace();
		}    	
	}

	public String getLastMessage() {
		return lastSentence;
	}

	public InetAddress getArduinoAddress() {
		return this.arduinoIpAddress;
	}

	public void stopThread() {
		this.running = false;
	}

	/**
	 * 
	 * @return the port of the UdpServer
	 */
	public int getPort() {
		return UDPPORT;
	}
	
	/**
	 * 
	 * @return the IP address of the current Machine
	 * @throws UnknownHostException
	 */
	public InetAddress getDeviceConnected() {
		return arduinoIpAddress;
	}
}
