package view;

import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.IOException;

import javax.swing.*;
import javax.swing.event.*;

import controller.ControllerManager;
import controller.Controller.PanelObserver;

@SuppressWarnings("serial")
public class FirstPanel extends JPanel implements PanelObserver{
	/**
	 * @author Giovanni Romio
	 * @author Battistini Luca
	 */

	/* Panel dimensions */
	private static final int WIDTH = 640;
	private static final int HEIGHT = 370;
	/* JTextField dimensions*/
	private static final int TXTWIDTH = 200;
	private static final int TXTHEIGHT = 20;
	/* JLabel dimensions*/
	private static final int LBLWIDTH = 200;
	private static final int LBLHEIGHT = 20;
	/*Ip address*/
	private static final String IP = "192.168.1.150";
	/*Components*/
	private JLabel lblDevices;
	private JLabel lblIpAdress;
	private JLabel lblTimer;
	private JTextField txtDevices;
	private JButton btnContinue;
	private JRadioButton rdbManual;
	private JRadioButton rdbRemote;
	private JLabel lblImage;
	/*MainFrame*/
	private MainFrame mainFrame;
	/*Controller*/
	private ControllerManager cm;
	/*Logo image*/
	private ImageIcon image = new ImageIcon(
			getClass().getResource("/logo4.png"));

	/**
	 * Initialize connection between UdpServer and Bomb UdpClient
	 * @throws IOException 
	 */
	public FirstPanel(ControllerManager controller){
		super();
		/*Controller manager*/
		cm = controller;
		/*Get instance of mainFrame*/
		mainFrame = MainFrame.getIstance();
		/*Size of the Panel*/
		this.setSize(new Dimension(WIDTH, HEIGHT));
		this.setPreferredSize(new Dimension(WIDTH, HEIGHT));

		lblDevices = new JLabel("Device connected:");
		lblDevices.setBounds(
				(int) (this.getWidth()/2 - lblDevices.getPreferredSize().getWidth()/2),
				(int) (this.getHeight()*0.3),
				(int)lblDevices.getPreferredSize().getWidth(),
				(int)lblDevices.getPreferredSize().getHeight()
				);

		txtDevices = new JTextField();
		txtDevices.setBounds(
				(int) (this.getWidth()/2 - TXTWIDTH/2),
				(int) (this.getHeight()*0.4),
				TXTWIDTH,
				TXTHEIGHT
				);
		/*Enable continue Button if connection between PC and ARDUINO is established*/
		txtDevices.getDocument().addDocumentListener(new DocumentListener() {
			@Override
			public void removeUpdate(DocumentEvent e) {

			}

			@Override
			public void insertUpdate(DocumentEvent e) {
				btnContinue.setEnabled(true);				
			}

			@Override
			public void changedUpdate(DocumentEvent e) {

			}
		});

		btnContinue = new JButton("Continue");
		btnContinue.setBounds(
				(int) (this.getWidth()/2 - btnContinue.getPreferredSize().getWidth()/2),
				(int) (this.getHeight()*0.5),
				(int)btnContinue.getPreferredSize().getWidth(),
				(int)btnContinue.getPreferredSize().getHeight()
				);
		btnContinue.setEnabled(false);
		btnContinue.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				setNextPanel();
			}
		});

		rdbManual = new JRadioButton("Manual");
		rdbManual.setBounds(
				(int)(this.getWidth()/2 - rdbManual.getPreferredSize().getWidth()),
				(int)(this.getHeight()*0.6),
				(int)rdbManual.getPreferredSize().getWidth(),
				(int)rdbManual.getPreferredSize().getHeight()
				);
		rdbManual.setSelected(true);

		rdbRemote = new JRadioButton("Remote");
		rdbRemote.setBounds(
				(int)(this.getWidth()/2),
				(int)(this.getHeight()*0.6),
				(int)rdbRemote.getPreferredSize().getWidth(),
				(int)rdbRemote.getPreferredSize().getHeight()
				);
		
		ButtonGroup radio = new ButtonGroup();
		radio.add(rdbManual);
		radio.add(rdbRemote);

		lblTimer = new JLabel();
		lblTimer.setBounds(
				(int) ((this.getWidth()*0.4)),
				(int)(this.getHeight()*0.7),
				LBLWIDTH,
				LBLHEIGHT
				);
		
		lblIpAdress = new JLabel("Ip:" + IP);
		lblIpAdress.setBounds(
				(int) (this.getWidth()*0.7),
				(int) (this.getHeight() - LBLHEIGHT),
				(int)lblIpAdress.getPreferredSize().getWidth(),
				(int)lblIpAdress.getPreferredSize().getHeight()
				);
		lblImage = new JLabel(image);
		lblImage.setBounds(0,0,image.getIconWidth(),image.getIconHeight());
				
		
		this.setLayout(null);
		
		this.add(lblImage);
		this.add(lblDevices);
		this.add(txtDevices);
		this.add(btnContinue);
		this.add(rdbManual);
		this.add(rdbRemote);
		this.add(lblIpAdress);
		this.add(lblTimer);

		cm.getCurrentController().attachObserver(this);
		
	}

	@Override
	public void displayIp(String ip) {
		this.txtDevices.setText(ip);
	}

	@Override
	public void displayUpdate(String message) {
		int i = Integer.parseInt(message);
		lblTimer.setText("MANUAL CONTROL IN:" + i/1000);
		mainFrame.repaint();			
	}

	@Override
	public void setNextPanel() {
		if(rdbManual.isSelected()){
			cm.getCurrentController().getServer().sendMessage("MANUAL");
			mainFrame.setContentPane(new ThirdPanel(cm));
		}else{
			cm.getCurrentController().getServer().sendMessage("REMOTE");
			mainFrame.setContentPane(new SecondPanel(cm));
		}
		mainFrame.repaint();
		
	}

}
