package view;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.*;
import javax.swing.border.Border;

import controller.Controller.PanelObserver;
import controller.ControllerManager;

@SuppressWarnings("serial")
public class SecondPanel extends JPanel implements PanelObserver{
	/**
	 * 
	 * @author Giovanni Romio, Battistini Luca
	 */

	/* Panel Dimensions */
	private static final int WIDTH = 640;
	private static final int HEIGHT = 370;
	/* JTextField dimensions*/
	private static final int TXTWIDTH = 230;
	private static final int TXTHEIGHT = 20;
	/*TextArea Dimension*/
	private static final int TXTAREAWIDTH = 300;
	private static final int TEXTAREAHEIGHT = 250;
	/*Ip address of current machine*/
	private static final String IP = "192.168.1.150";
	/* SubPanels*/
	private JPanel leftPanel;
	private JPanel rightPanel;
	/* Components of the Frame*/
	private JLabel arduinoIp;
	private JLabel pcIp;
	private JTextField txtPcMessage;
	private JTextArea arduinoLog;
	private JTextArea pcLog;
	private JButton btnSend;
	/*MainFrame*/
	private MainFrame mainFrame;
	private ControllerManager cm;

	public SecondPanel(ControllerManager controller) {
		super();
		cm = controller;
		mainFrame = MainFrame.getIstance();
		/*Size of the Panel*/
		this.setSize(new Dimension(WIDTH, HEIGHT));
		this.setPreferredSize(new Dimension(WIDTH, HEIGHT));
		this.setLayout(new BorderLayout());

		/*Initialize subPanels*/
		leftPanel = new JPanel();
		leftPanel.setLayout(new FlowLayout());
		leftPanel.setPreferredSize(new Dimension(WIDTH/2, HEIGHT));
		rightPanel = new JPanel();
		rightPanel.setLayout(new FlowLayout());
		rightPanel.setPreferredSize(new Dimension(WIDTH/2, HEIGHT));

		/*Border of the JText area*/
		Border border = BorderFactory.createLineBorder(Color.BLACK);

		/*Labels containing ip address of Arduino and pc*/
		arduinoIp = new JLabel("arduino Ip:");
		pcIp = new JLabel("pc Ip:" + IP);

		/*Text area containing messages received from Arduino*/
		arduinoLog = new JTextArea();
		arduinoLog.setPreferredSize(new Dimension(TXTAREAWIDTH, TEXTAREAHEIGHT));
		arduinoLog.setBorder(BorderFactory.createCompoundBorder(border, 
				BorderFactory.createEmptyBorder(10, 10, 10, 10)));

		pcLog = new JTextArea();
		pcLog.setPreferredSize(new Dimension(TXTAREAWIDTH, TEXTAREAHEIGHT));
		pcLog.setBorder(BorderFactory.createCompoundBorder(border, 
				BorderFactory.createEmptyBorder(10, 10, 10, 10)));

		/*Text field to enter the message to send to Arduino*/
		txtPcMessage = new JTextField();
		txtPcMessage.setPreferredSize(new Dimension(TXTWIDTH, TXTHEIGHT));

		/*Button to send message to Arduino via Udp connection*/
		btnSend = new JButton("Send");
		btnSend.addActionListener(new ActionListener() {

			@Override
			public void actionPerformed(ActionEvent e) {
				pcLog.append(txtPcMessage.getText() + "\n");
				/*Notifica il controllore ed agisce sul modello*/
				cm.getCurrentController().setBomb(txtPcMessage.getText());				
			}
		});

		leftPanel.add(arduinoIp);
		leftPanel.add(arduinoLog);

		rightPanel.add(pcIp);
		rightPanel.add(pcLog);
		rightPanel.add(txtPcMessage);
		rightPanel.add(btnSend);

		this.add(leftPanel, BorderLayout.WEST);
		this.add(rightPanel, BorderLayout.EAST);

		/*Set the currentController at 2nd elements*/
		cm.setController(1);
		/*attach the controller to the panel*/
		cm.getCurrentController().attachObserver(this);

	}
	
	/**
	 * Display the Ip of the Device Connected 
	 */
	@Override
	public void displayIp(String ip) {
		arduinoIp.setText("arduino Ip:" + ip );
	}
	
	/**
	 * Display the current Message from Arduino
	 */
	@Override
	public void displayUpdate(String message) {
		arduinoLog.append(message + "\n");
	}
	
	/**
	 * Switch to 3rd Panel after Timer is set 
	 */
	@Override
	public void setNextPanel() {		
		mainFrame.setContentPane(new ThirdPanel(cm));
		mainFrame.repaint();
	}

}
